@@ -26,7 +26,9 @@
 
 /**
  * SECTION:clutter-behaviour-ellipse
+ * @Title: ClutterBehaviourEllipse
  * @short_description: A behaviour interpolating position along an ellipse
+ * @Deprecated: 1.6: Use clutter_actor_animate() instead
  *
  * #ClutterBehaviourEllipse interpolates actors along a path defined by
  *  an ellipse.
@@ -42,16 +44,16 @@
 #include "config.h"
 #endif
 
-#include "clutter-fixed.h"
-#include "clutter-marshal.h"
+#include <math.h>
+#include <stdlib.h>
+
+#undef CLUTTER_DISABLE_DEPRECATED
 #include "clutter-behaviour-ellipse.h"
+
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-private.h"
 
-#include <stdlib.h>
-#include <memory.h>
-
 G_DEFINE_TYPE (ClutterBehaviourEllipse,
                clutter_behaviour_ellipse,
 	       CLUTTER_TYPE_BEHAVIOUR);
@@ -548,7 +550,7 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
 
 /**
  * clutter_behaviour_ellipse_new:
- * @alpha: a #ClutterAlpha, or %NULL
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @x: x coordinace of the center
  * @y: y coordiance of the center
  * @width: width of the ellipse
@@ -564,6 +566,10 @@ clutter_behaviour_ellipse_init (ClutterBehaviourEllipse * self)
  * interval <0, 360); if @start is equal to @end, the behaviour will
  * rotate by exacly 360 degrees.
  *
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
  * Return value: the newly created #ClutterBehaviourEllipse
  *
  * Since: 0.4
@@ -629,8 +635,8 @@ clutter_behaviour_ellipse_set_center (ClutterBehaviourEllipse *self,
 /**
  * clutter_behaviour_ellipse_get_center:
  * @self: a #ClutterBehaviourEllipse
- * @x: return location for the X coordinate of the center, or %NULL
- * @y: return location for the Y coordinate of the center, or %NULL
+ * @x: (out): return location for the X coordinate of the center, or %NULL
+ * @y: (out): return location for the Y coordinate of the center, or %NULL
  *
  * Gets the center of the elliptical path path.
  *
@@ -979,9 +985,9 @@ clutter_behaviour_ellipse_set_tilt (ClutterBehaviourEllipse *self,
 /**
  * clutter_behaviour_ellipse_get_tilt:
  * @self: a #ClutterBehaviourEllipse
- * @angle_tilt_x: return location for tilt angle on the X axis, or %NULL.
- * @angle_tilt_y: return location for tilt angle on the Y axis, or %NULL.
- * @angle_tilt_z: return location for tilt angle on the Z axis, or %NULL.
+ * @angle_tilt_x: (out): return location for tilt angle on the X axis, or %NULL.
+ * @angle_tilt_y: (out): return location for tilt angle on the Y axis, or %NULL.
+ * @angle_tilt_z: (out): return location for tilt angle on the Z axis, or %NULL.
  *
  * Gets the tilt of the ellipse around the center in Y axis.
  *
