@@ -591,7 +591,9 @@ clutter_box_set_layout_manager (ClutterBox           *box,
  *
  * Retrieves the #ClutterLayoutManager instance used by @box
  *
- * Return value: a #ClutterLayoutManager
+ * Return value: (transfer none): a #ClutterLayoutManager. The returned
+ *   #ClutterLayoutManager is owned by the #ClutterBox and it should not
+ *   be unreferenced
  *
  * Since: 1.2
  */
@@ -736,14 +738,9 @@ clutter_box_set_property_valist (ClutterBox   *box,
           break;
         }
 
-#if GLIB_CHECK_VERSION (2, 23, 2)
       G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
                             var_args, 0,
                             &error);
-#else
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, var_args, 0, &error);
-#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
 
       if (error)
         {
