@@ -34,8 +34,10 @@
 #include "config.h"
 #endif
 
-#include "clutter-debug.h"
+#undef CLUTTER_DISABLE_DEPRECATED
 #include "clutter-timeout-pool.h"
+
+#include "clutter-debug.h"
 #include "clutter-timeout-interval.h"
 
 typedef struct _ClutterTimeout  ClutterTimeout;
@@ -63,8 +65,9 @@ struct _ClutterTimeoutPool
 
   guint next_id;
 
-  GTimeVal start_time;
-  GList *timeouts, *dispatched_timeouts;
+  GList *timeouts;
+  GList *dispatched_timeouts;
+
   gint ready;
 
   guint id;
@@ -133,11 +136,21 @@ clutter_timeout_prepare (ClutterTimeoutPool *pool,
                          ClutterTimeout     *timeout,
                          gint               *next_timeout)
 {
-  GTimeVal now;
+  GSource *source = (GSource *) pool;
+  gint64 now;
 
-  g_source_get_current_time (&pool->source, &now);
+#if GLIB_CHECK_VERSION (2, 27, 3)
+  now = g_source_get_time (source) / 1000;
+#else
+  {
+    GTimeVal source_time;
+    g_source_get_current_time (source, &source_time);
+    now = source_time.tv_sec * 1000 + source_time.tv_usec / 1000;
+  }
+#endif
 
-  return _clutter_timeout_interval_prepare (&now, &timeout->interval,
+  return _clutter_timeout_interval_prepare (now,
+                                            &timeout->interval,
                                             next_timeout);
 }
 
@@ -365,6 +378,8 @@ clutter_timeout_pool_finalize (GSource *source)
  *   the destruction of the timeout pool using g_source_destroy()
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 ClutterTimeoutPool *
 clutter_timeout_pool_new (gint priority)
@@ -377,16 +392,12 @@ clutter_timeout_pool_new (gint priority)
   if (!source)
     return NULL;
 
-#if GLIB_CHECK_VERSION (2, 25, 8)
   g_source_set_name (source, "Clutter timeout pool");
-#endif
 
   if (priority != G_PRIORITY_DEFAULT)
     g_source_set_priority (source, priority);
 
   pool = (ClutterTimeoutPool *) source;
-
-  g_get_current_time (&pool->start_time);
   pool->next_id = 1;
   pool->id = g_source_attach (source, NULL);
 
@@ -423,6 +434,8 @@ clutter_timeout_pool_new (gint priority)
  *   Use clutter_timeout_pool_remove() to stop the timeout.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 guint
 clutter_timeout_pool_add (ClutterTimeoutPool *pool,
@@ -458,6 +471,8 @@ clutter_timeout_pool_add (ClutterTimeoutPool *pool,
  * clutter_timeout_pool_add().
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 void
 clutter_timeout_pool_remove (ClutterTimeoutPool *pool,
