@@ -39,6 +39,18 @@
 
 G_BEGIN_DECLS
 
+#ifndef CLUTTER_DISABLE_DEPRECATED
+
+/**
+ * ClutterTimeoutPool: (skip)
+ *
+ * <structname>ClutterTimeoutPool</structname> is an opaque structure
+ * whose members cannot be directly accessed.
+ *
+ * Since: 0.6
+ *
+ * Deprecated: 1.6
+ */
 typedef struct _ClutterTimeoutPool    ClutterTimeoutPool;
 
 ClutterTimeoutPool *clutter_timeout_pool_new    (gint                priority);
@@ -50,6 +62,8 @@ guint               clutter_timeout_pool_add    (ClutterTimeoutPool *pool,
 void                clutter_timeout_pool_remove (ClutterTimeoutPool *pool,
                                                  guint               id);
 
+#endif /* CLUTTER_DISABLE_DEPRECATED */
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TIMEOUT_POOL_H__ */
