@@ -29,7 +29,6 @@
 #define __CLUTTER_TYPES_H__
 
 #include <glib-object.h>
-#include <clutter/clutter-units.h>
 
 G_BEGIN_DECLS
 
@@ -52,6 +51,11 @@ typedef struct _ClutterAnimator         ClutterAnimator;
 
 typedef struct _ClutterAction           ClutterAction;
 typedef struct _ClutterConstraint       ClutterConstraint;
+typedef struct _ClutterEffect           ClutterEffect;
+
+typedef struct _ClutterShader           ClutterShader;
+
+typedef struct _ClutterColor            ClutterColor;
 
 typedef union _ClutterEvent             ClutterEvent;
 
@@ -189,6 +193,13 @@ void             clutter_actor_box_union          (const ClutterActorBox *a,
                                                    const ClutterActorBox *b,
                                                    ClutterActorBox       *result);
 
+void             clutter_actor_box_set_origin     (ClutterActorBox       *box,
+                                                   gfloat                 x,
+                                                   gfloat                 y);
+void             clutter_actor_box_set_size       (ClutterActorBox       *box,
+                                                   gfloat                 width,
+                                                   gfloat                 height);
+
 /**
  * ClutterGeometry:
  * @x: X coordinate of the top left corner of an actor
