@@ -28,20 +28,14 @@
 #include "clutter-glx.h"
 #include "clutter-profile.h"
 
-#include "../clutter-main.h"
-#include "../clutter-feature.h"
-#include "../clutter-color.h"
-#include "../clutter-util.h"
-#include "../clutter-event.h"
-#include "../clutter-enum-types.h"
-#include "../clutter-private.h"
-#include "../clutter-debug.h"
-#include "../clutter-units.h"
-#include "../clutter-shader.h"
-#include "../clutter-group.h"
-#include "../clutter-container.h"
-#include "../clutter-stage.h"
-#include "../clutter-stage-window.h"
+#include "clutter-actor-private.h"
+#include "clutter-debug.h"
+#include "clutter-event.h"
+#include "clutter-enum-types.h"
+#include "clutter-feature.h"
+#include "clutter-main.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 #include "cogl/cogl.h"
 
@@ -297,9 +291,16 @@ clutter_stage_glx_has_redraw_clips (ClutterStageWindow *stage_window)
 {
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
 
-  /* NB: a degenerate clip means a full stage redraw is required */
-  if (stage_glx->initialized_redraw_clip &&
-      stage_glx->bounding_redraw_clip.width != 0)
+  /* NB: at the start of each new frame there is an implied clip that
+   * clips everything (i.e. nothing would be drawn) so we need to make
+   * sure we return True in the un-initialized case here.
+   *
+   * NB: a clip width of 0 means a full stage redraw has been queued
+   * so we effectively don't have any redraw clips in that case.
+   */
+  if (!stage_glx->initialized_redraw_clip ||
+      (stage_glx->initialized_redraw_clip &&
+       stage_glx->bounding_redraw_clip.width != 0))
     return TRUE;
   else
     return FALSE;
@@ -310,7 +311,7 @@ clutter_stage_glx_ignoring_redraw_clips (ClutterStageWindow *stage_window)
 {
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
 
-  /* NB: a degenerate clip means a full stage redraw is required */
+  /* NB: a clip width of 0 means a full stage redraw is required */
   if (stage_glx->initialized_redraw_clip &&
       stage_glx->bounding_redraw_clip.width == 0)
     return TRUE;
@@ -375,7 +376,7 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
     return;
 
   /* A NULL stage clip means a full stage redraw has been queued and
-   * we keep track of this by setting a degenerate
+   * we keep track of this by setting a zero width
    * stage_glx->bounding_redraw_clip */
   if (stage_clip == NULL)
     {
@@ -414,7 +415,7 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
   if (redraw_area > (stage_area * 0.7f))
     {
       g_print ("DEBUG: clipped redraw too big, forcing full redraw\n");
-      /* Set a degenerate clip to force a full redraw */
+      /* Set a zero width clip to force a full redraw */
       stage_glx->bounding_redraw_clip.width = 0;
     }
 #endif
@@ -498,6 +499,8 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
   ClutterStageX11   *stage_x11;
   GLXDrawable        drawable;
   unsigned int       video_sync_count;
+  gboolean           may_use_clipped_redraw;
+  gboolean           use_clipped_redraw;
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
                         "Painting actors",
@@ -522,21 +525,84 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
-  if (backend_glx->can_blit_sub_buffer &&
-      /* NB: a degenerate redraw clip width == full stage redraw */
-      (stage_glx->bounding_redraw_clip.width != 0) &&
+  if (G_LIKELY (backend_glx->can_blit_sub_buffer) &&
+      /* NB: a zero width redraw clip == full stage redraw */
+      stage_glx->bounding_redraw_clip.width != 0 &&
+      /* some drivers struggle to get going and produce some junk
+       * frames when starting up... */
+      G_LIKELY (stage_glx->frame_count > 3) &&
+      /* While resizing a window clipped redraws are disabled to avoid
+       * artefacts. See clutter-event-x11.c:event_translate for a
+       * detailed explanation */
+      G_LIKELY (stage_x11->clipped_redraws_cool_off == 0))
+    may_use_clipped_redraw = TRUE;
+  else
+    may_use_clipped_redraw = FALSE;
+
+  if (may_use_clipped_redraw &&
       G_LIKELY (!(clutter_paint_debug_flags &
                   CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS)))
+    use_clipped_redraw = TRUE;
+  else
+    use_clipped_redraw = FALSE;
+
+  if (use_clipped_redraw)
     {
       cogl_clip_push_window_rectangle (stage_glx->bounding_redraw_clip.x,
                                        stage_glx->bounding_redraw_clip.y,
                                        stage_glx->bounding_redraw_clip.width,
                                        stage_glx->bounding_redraw_clip.height);
-      clutter_actor_paint (CLUTTER_ACTOR (stage));
+      _clutter_stage_do_paint (stage, &stage_glx->bounding_redraw_clip);
       cogl_clip_pop ();
     }
   else
-    clutter_actor_paint (CLUTTER_ACTOR (stage));
+    _clutter_stage_do_paint (stage, NULL);
+
+  if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS &&
+      may_use_clipped_redraw)
+    {
+      ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
+      static CoglMaterial *outline = NULL;
+      CoglHandle vbo;
+      float x_1 = clip->x;
+      float x_2 = clip->x + clip->width;
+      float y_1 = clip->y;
+      float y_2 = clip->y + clip->height;
+      float quad[8] = {
+        x_1, y_1,
+        x_2, y_1,
+        x_2, y_2,
+        x_1, y_2
+      };
+      CoglMatrix modelview;
+
+      if (outline == NULL)
+        {
+          outline = cogl_material_new ();
+          cogl_material_set_color4ub (outline, 0xff, 0x00, 0x00, 0xff);
+        }
+
+      vbo = cogl_vertex_buffer_new (4);
+      cogl_vertex_buffer_add (vbo,
+                              "gl_Vertex",
+                              2, /* n_components */
+                              COGL_ATTRIBUTE_TYPE_FLOAT,
+                              FALSE, /* normalized */
+                              0, /* stride */
+                              quad);
+      cogl_vertex_buffer_submit (vbo);
+
+      cogl_push_matrix ();
+      cogl_matrix_init_identity (&modelview);
+      _clutter_actor_apply_modelview_transform (CLUTTER_ACTOR (stage),
+                                                &modelview);
+      cogl_set_modelview_matrix (&modelview);
+      cogl_set_source (outline);
+      cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
+                               0 , 4);
+      cogl_pop_matrix ();
+      cogl_object_unref (vbo);
+    }
 
   cogl_flush ();
   CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
@@ -558,11 +624,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
     backend_glx->get_video_sync (&video_sync_count);
 
   /* push on the screen */
-  if (backend_glx->can_blit_sub_buffer &&
-      /* NB: a degenerate redraw clip width == full stage redraw */
-      (stage_glx->bounding_redraw_clip.width != 0) &&
-      G_LIKELY (!(clutter_paint_debug_flags &
-                  CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS)))
+  if (use_clipped_redraw)
     {
       ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
       ClutterGeometry copy_area;
@@ -577,43 +639,6 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                     stage_glx->bounding_redraw_clip.width,
                     stage_glx->bounding_redraw_clip.height);
 
-      if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)
-        {
-          static CoglHandle outline = COGL_INVALID_HANDLE;
-          CoglHandle vbo;
-          float x_1 = clip->x;
-          float x_2 = clip->x + clip->width;
-          float y_1 = clip->y;
-          float y_2 = clip->y + clip->height;
-          float quad[8] = {
-            x_1, y_1,
-            x_2, y_1,
-            x_2, y_2,
-            x_1, y_2
-          };
-
-          if (outline == COGL_INVALID_HANDLE)
-            {
-              outline = cogl_material_new ();
-              cogl_material_set_color4ub (outline, 0xff, 0x00, 0x00, 0xff);
-            }
-
-          vbo = cogl_vertex_buffer_new (4);
-          cogl_vertex_buffer_add (vbo,
-                                  "gl_Vertex",
-                                  2, /* n_components */
-                                  COGL_ATTRIBUTE_TYPE_FLOAT,
-                                  FALSE, /* normalized */
-                                  0, /* stride */
-                                  quad);
-          cogl_vertex_buffer_submit (vbo);
-
-          cogl_set_source (outline);
-          cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
-                                   0 , 4);
-          cogl_flush ();
-        }
-
       /* XXX: It seems there will be a race here in that the stage
        * window may be resized before glXCopySubBufferMESA is handled
        * and so we may copy the wrong region. I can't really see how
@@ -718,5 +743,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   /* reset the redraw clipping for the next paint... */
   stage_glx->initialized_redraw_clip = FALSE;
+
+  stage_glx->frame_count++;
 }
 
