@@ -30,12 +30,13 @@
 #include "clutter-win32.h"
 #include "clutter-device-manager-win32.h"
 
-#include "../clutter-event.h"
-#include "../clutter-main.h"
-#include "../clutter-input-device.h"
-#include "../clutter-debug.h"
-#include "../clutter-private.h"
-#include "../clutter-version.h"
+#include "clutter-event.h"
+#include "clutter-main.h"
+#include "clutter-device-manager-private.h"
+#include "clutter-debug.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
+#include "clutter-version.h"
 
 #include "cogl/cogl.h"
 
@@ -51,6 +52,9 @@ static gchar *clutter_vblank_name = NULL;
 
 static HINSTANCE clutter_hinst = NULL;
 
+/* various flags corresponding to pre init setup calls */
+static gboolean _no_event_retrieval = FALSE;
+
 gboolean
 clutter_backend_win32_pre_parse (ClutterBackend  *backend,
 				 GError         **error)
@@ -75,7 +79,8 @@ clutter_backend_win32_init_events (ClutterBackend *backend)
                   "backend", backend_win32,
                   NULL);
 
-  _clutter_backend_win32_events_init (backend);
+  if (!_no_event_retrieval)
+    _clutter_backend_win32_events_init (backend);
 }
 
 HCURSOR
@@ -125,14 +130,9 @@ clutter_backend_win32_dispose (GObject *gobject)
   ClutterStageManager *stage_manager;
 
   CLUTTER_NOTE (BACKEND, "Disposing the of stages");
-
   stage_manager = clutter_stage_manager_get_default ();
 
-  /* Destroy all of the stages. g_slist_foreach is used because the
-     finalizer for the stages will remove the stage from the
-     stage_manager's list and g_slist_foreach has some basic
-     protection against this */
-  g_slist_foreach (stage_manager->stages, (GFunc) clutter_actor_destroy, NULL);
+  g_object_unref (stage_manager);
 
   CLUTTER_NOTE (BACKEND, "Removing the event source");
   _clutter_backend_win32_events_uninit (CLUTTER_BACKEND (backend_win32));
@@ -539,6 +539,29 @@ clutter_backend_win32_get_device_manager (ClutterBackend *backend)
   return backend_win32->device_manager;
 }
 
+/**
+ * clutter_win32_disable_event_retrieval
+ *
+ * Disables retrieval of Windows messages in the main loop. Use to
+ * create event-less canvas.
+ *
+ * This function can only be called before calling clutter_init().
+ *
+ * Since: 0.8
+ */
+void
+clutter_win32_disable_event_retrieval (void)
+{
+  if (_clutter_context_is_initialized ())
+    {
+      g_warning ("clutter_win32_disable_event_retrieval() can only be "
+                 "called before clutter_init()");
+      return;
+    }
+
+  _no_event_retrieval = TRUE;
+}
+
 static void
 clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
 {
@@ -563,10 +586,7 @@ clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
 static void
 clutter_backend_win32_init (ClutterBackendWin32 *backend_win32)
 {
-  ClutterBackend *backend = CLUTTER_BACKEND (backend_win32);
-
   backend_win32->gl_context         = NULL;
-  backend_win32->no_event_retrieval = FALSE;
   backend_win32->invisible_cursor   = NULL;
 
   /* FIXME: get from GetSystemMetric?
