@@ -19,8 +19,15 @@ typedef struct _TestConformSimpleFixture
   int dummy;
 } TestConformSimpleFixture;
 
+typedef struct _TestConformTodo
+{
+  gchar *name;
+  void (* func) (TestConformSimpleFixture *, gconstpointer);
+} TestConformTodo;
+
 void test_conform_simple_fixture_setup (TestConformSimpleFixture *fixture,
 					gconstpointer data);
 void test_conform_simple_fixture_teardown (TestConformSimpleFixture *fixture,
 					   gconstpointer data);
 
+gchar *clutter_test_get_data_file (const gchar *filename);
