@@ -207,23 +207,23 @@ key_group_class_init (KeyGroupClass *klass)
   binding_pool = clutter_binding_pool_get_for_class (klass);
 
   clutter_binding_pool_install_action (binding_pool, "move-right",
-                                       CLUTTER_Right, 0,
+                                       CLUTTER_KEY_Right, 0,
                                        G_CALLBACK (key_group_action_move_right),
                                        NULL, NULL);
   clutter_binding_pool_install_action (binding_pool, "move-left",
-                                       CLUTTER_Left, 0,
+                                       CLUTTER_KEY_Left, 0,
                                        G_CALLBACK (key_group_action_move_left),
                                        NULL, NULL);
   clutter_binding_pool_install_action (binding_pool, "activate",
-                                       CLUTTER_Return, 0,
+                                       CLUTTER_KEY_Return, 0,
                                        G_CALLBACK (key_group_action_activate),
                                        NULL, NULL);
   clutter_binding_pool_install_action (binding_pool, "activate",
-                                       CLUTTER_KP_Enter, 0,
+                                       CLUTTER_KEY_KP_Enter, 0,
                                        G_CALLBACK (key_group_action_activate),
                                        NULL, NULL);
   clutter_binding_pool_install_action (binding_pool, "activate",
-                                       CLUTTER_ISO_Enter, 0,
+                                       CLUTTER_KEY_ISO_Enter, 0,
                                        G_CALLBACK (key_group_action_activate),
                                        NULL, NULL);
 }
@@ -245,9 +245,6 @@ G_MODULE_EXPORT int
 test_binding_pool_main (int argc, char *argv[])
 {
   ClutterActor *stage, *key_group;
-  ClutterColor red_color   = { 255,   0,   0, 255 };
-  ClutterColor green_color = {   0, 255,   0, 255 };
-  ClutterColor blue_color  = {   0,   0, 255, 255 };
   gint group_x, group_y;
 
   clutter_init (&argc, &argv);
@@ -263,21 +260,21 @@ test_binding_pool_main (int argc, char *argv[])
   /* add three rectangles to the key group */
   clutter_container_add (CLUTTER_CONTAINER (key_group),
                          g_object_new (CLUTTER_TYPE_RECTANGLE,
-                                       "color", &red_color,
+                                       "color", CLUTTER_COLOR_Red,
                                        "width", 50.0,
                                        "height", 50.0,
                                        "x", 0.0,
                                        "y", 0.0,
                                        NULL),
                          g_object_new (CLUTTER_TYPE_RECTANGLE,
-                                       "color", &green_color,
+                                       "color", CLUTTER_COLOR_Green,
                                        "width", 50.0,
                                        "height", 50.0,
                                        "x", 75.0,
                                        "y", 0.0,
                                        NULL),
                          g_object_new (CLUTTER_TYPE_RECTANGLE,
-                                       "color", &blue_color,
+                                       "color", CLUTTER_COLOR_Blue,
                                        "width", 50.0,
                                        "height", 50.0,
                                        "x", 150.0,
